package dal

import (
	"context"

	"ifms/internal/mods/autocontrol/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get device type info storage instance
func GetDeviceTypeInfoDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.DeviceTypeInfo))
}

// 设备类型信息
type DeviceTypeInfo struct {
	DB *gorm.DB
}

// Query device type infos from the database based on the provided parameters and options.
func (a *DeviceTypeInfo) Query(ctx context.Context, params schema.DeviceTypeInfoQueryParam, opts ...schema.DeviceTypeInfoQueryOptions) (*schema.DeviceTypeInfoQueryResult, error) {
	var opt schema.DeviceTypeInfoQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	db := GetDeviceTypeInfoDB(ctx, a.DB)

	var list schema.DeviceTypeInfos
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.DeviceTypeInfoQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified device type info from the database.
func (a *DeviceTypeInfo) Get(ctx context.Context, id string, opts ...schema.DeviceTypeInfoQueryOptions) (*schema.DeviceTypeInfo, error) {
	var opt schema.DeviceTypeInfoQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.DeviceTypeInfo)
	ok, err := util.FindOne(ctx, GetDeviceTypeInfoDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified device type info exists in the database.
func (a *DeviceTypeInfo) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetDeviceTypeInfoDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new device type info.
func (a *DeviceTypeInfo) Create(ctx context.Context, item *schema.DeviceTypeInfo) error {
	result := GetDeviceTypeInfoDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified device type info in the database.
func (a *DeviceTypeInfo) Update(ctx context.Context, item *schema.DeviceTypeInfo) error {
	result := GetDeviceTypeInfoDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified device type info from the database.
func (a *DeviceTypeInfo) Delete(ctx context.Context, id string) error {
	result := GetDeviceTypeInfoDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.DeviceTypeInfo))
	return errors.WithStack(result.Error)
}
