package autocontrol

import (
	"context"
	"ifms/internal/config"

	"ifms/internal/mods/autocontrol/api"
	"ifms/internal/mods/autocontrol/schema"

	"github.com/gin-gonic/gin"
	"gorm.io/gorm"
)

type AutoControl struct {
	DB                  *gorm.DB
	DeviceRunLogAPI     *api.DeviceRunLog
	DeviceControlLogAPI *api.DeviceControlLog
	DeviceAPI           *api.Device
	DeviceFishPondAPI   *api.DeviceFishPond
	DeviceTypeInfoAPI   *api.DeviceTypeInfo
}

func (a *AutoControl) AutoMigrate(ctx context.Context) error {
	return a.DB.AutoMigrate(new(schema.DeviceRunLog), new(schema.DeviceControlLog), new(schema.Device), new(schema.DeviceFishPond), new(schema.DeviceTypeInfo))
}

func (a *AutoControl) Init(ctx context.Context) error {
	if config.C.Storage.DB.AutoMigrate {
		if err := a.AutoMigrate(ctx); err != nil {
			return err
		}
	}
	return nil
}

func (a *AutoControl) RegisterV1Routers(ctx context.Context, v1 *gin.RouterGroup) error {
	v1 = v1.Group("autocontrol")
	deviceRunLog := v1.Group("device-run-logs")
	{
		deviceRunLog.GET("", a.DeviceRunLogAPI.Query)
		deviceRunLog.GET(":id", a.DeviceRunLogAPI.Get)
		deviceRunLog.POST("", a.DeviceRunLogAPI.Create)
		// deviceRunLog.PUT(":id", a.DeviceRunLogAPI.Update)
		// deviceRunLog.DELETE(":id", a.DeviceRunLogAPI.Delete)
	}
	deviceControlLog := v1.Group("device-control-logs")
	{
		deviceControlLog.GET("", a.DeviceControlLogAPI.Query)
		deviceControlLog.GET(":id", a.DeviceControlLogAPI.Get)
		deviceControlLog.POST("", a.DeviceControlLogAPI.Create)
		// deviceControlLog.PUT(":id", a.DeviceControlLogAPI.Update)
		// deviceControlLog.DELETE(":id", a.DeviceControlLogAPI.Delete)
	}
	device := v1.Group("devices")
	{
		device.GET("", a.DeviceAPI.Query)
		device.GET(":id", a.DeviceAPI.Get)
		device.POST("", a.DeviceAPI.Create)
		device.PUT(":id", a.DeviceAPI.Update)
		// device.DELETE(":id", a.DeviceAPI.Delete)
		// 统计设备数量以及状态
		device.GET("statistics", a.DeviceAPI.Statistics)

		// 增氧机配置
		device.POST("aeration", a.DeviceAPI.AerationConfig)
		// 投喂机配置
		device.POST("feeding", a.DeviceAPI.FeedingConfig)
		// 水循环配置
		device.POST("water-circulation", a.DeviceAPI.WaterCirculationConfig)
	}
	deviceTypeInfo :=
		// deviceFishPond := v1.Group("device-fish-ponds")
		// {
		// 	deviceFishPond.GET("", a.DeviceFishPondAPI.Query)
		// 	deviceFishPond.GET(":id", a.DeviceFishPondAPI.Get)
		// 	deviceFishPond.POST("", a.DeviceFishPondAPI.Create)
		// 	deviceFishPond.PUT(":id", a.DeviceFishPondAPI.Update)
		// 	deviceFishPond.DELETE(":id", a.DeviceFishPondAPI.Delete)
		// }
		v1.Group("device-type-infos")
	{
		deviceTypeInfo.GET("", a.DeviceTypeInfoAPI.Query)
		deviceTypeInfo.GET(":id", a.DeviceTypeInfoAPI.Get)
		deviceTypeInfo.POST("", a.DeviceTypeInfoAPI.Create)
		deviceTypeInfo.PUT(":id", a.DeviceTypeInfoAPI.Update)
		deviceTypeInfo.DELETE(":id", a.DeviceTypeInfoAPI.Delete)
	}

	return nil
}

func (a *AutoControl) Release(ctx context.Context) error {
	return nil
}
