package schema

import (
	"time"

	"ifms/pkg/util"
)

// 设备-鱼池关联表
type DeviceFishPond struct {
	ID           int64     `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`                      // Unique ID
	TenantID     string    `json:"tenant_id" gorm:"size:32;"`                                            // Tenant ID
	DeviceID     int64     `json:"device_id" gorm:"foreignKey:ID;references:device;comment:设备ID;"`       // 设备ID
	FishPondID   int64     `json:"fish_pond_id" gorm:"foreignKey:ID;references:fish_pond;comment:鱼池ID;"` // 鱼池ID
	FishPondName string    `json:"fish_pond_name" gorm:"comment:鱼池名称;"`                                  // 鱼池名称
	CreatedAt    time.Time `json:"created_at" gorm:"comment:Create time;"`                               // Create time
	UpdatedAt    time.Time `json:"updated_at" gorm:"comment:Update time;"`                               // Update time
}

// Defining the query parameters for the `DeviceFishPond` struct.
type DeviceFishPondQueryParam struct {
	util.PaginationParam
}

// Defining the query options for the `DeviceFishPond` struct.
type DeviceFishPondQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `DeviceFishPond` struct.
type DeviceFishPondQueryResult struct {
	Data       DeviceFishPonds
	PageResult *util.PaginationResult
}

// Defining the slice of `DeviceFishPond` struct.
type DeviceFishPonds []*DeviceFishPond

// Defining the data structure for creating a `DeviceFishPond` struct.
type DeviceFishPondForm struct {
	TenantID     string `json:"tenant_id"`                         // 租户ID
	DeviceID     int64  `json:"device_id" binding:"required"`      // 设备ID
	FishPondID   int64  `json:"fish_pond_id" binding:"required"`   // 鱼池ID
	FishPondName string `json:"fish_pond_name" binding:"required"` // 鱼池名称
}

// A validation function for the `DeviceFishPondForm` struct.
func (a *DeviceFishPondForm) Validate() error {
	return nil
}

// Convert `DeviceFishPondForm` to `DeviceFishPond` object.
func (a *DeviceFishPondForm) FillTo(deviceFishPond *DeviceFishPond) error {
	deviceFishPond.TenantID = a.TenantID
	deviceFishPond.DeviceID = a.DeviceID
	deviceFishPond.FishPondID = a.FishPondID
	deviceFishPond.FishPondName = a.FishPondName
	return nil
}
