package schema

import (
	"time"

	"ifms/pkg/util"
)

// 设备类型信息
type DeviceTypeInfo struct {
	ID          int64     `json:"id" gorm:"primaryKey;comment:Unique ID;"`      // Unique ID
	Type        int       `json:"type" gorm:"comment:设备类型（1增氧机、2投喂机、3水循环）;"`    // 设备类型（1增氧机、2投喂机、3水循环）
	Model       string    `json:"model" gorm:"size:255;comment:设备型号;"`          // 设备型号
	Supplier    string    `json:"supplier" gorm:"size:255;comment:供应商;"`        // 供应商
	Pic         string    `json:"pic" gorm:"type:text;comment:图片地址;"`           // 图片地址
	Description string    `json:"description" gorm:"type:text;comment:简介;"`     // 简介
	CreatedAt   time.Time `json:"created_at" gorm:"index;comment:Create time;"` // Create time
	UpdatedAt   time.Time `json:"updated_at" gorm:"index;comment:Update time;"` // Update time
}

// Defining the query parameters for the `DeviceTypeInfo` struct.
type DeviceTypeInfoQueryParam struct {
	util.PaginationParam
}

// Defining the query options for the `DeviceTypeInfo` struct.
type DeviceTypeInfoQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `DeviceTypeInfo` struct.
type DeviceTypeInfoQueryResult struct {
	Data       DeviceTypeInfos
	PageResult *util.PaginationResult
}

// Defining the slice of `DeviceTypeInfo` struct.
type DeviceTypeInfos []*DeviceTypeInfo

// Defining the data structure for creating a `DeviceTypeInfo` struct.
type DeviceTypeInfoForm struct {
	Type        int    `json:"type" binding:"required"`             // 设备类型（1增氧机、2投喂机、3水循环）
	Model       string `json:"model" binding:"required,max=255"`    // 设备型号
	Supplier    string `json:"supplier" binding:"required,max=255"` // 供应商
	Pic         string `json:"pic" binding:"required"`              // 图片地址
	Description string `json:"description" binding:"required"`      // 简介
}

// A validation function for the `DeviceTypeInfoForm` struct.
func (a *DeviceTypeInfoForm) Validate() error {
	return nil
}

// Convert `DeviceTypeInfoForm` to `DeviceTypeInfo` object.
func (a *DeviceTypeInfoForm) FillTo(deviceTypeInfo *DeviceTypeInfo) error {
	deviceTypeInfo.Type = a.Type
	deviceTypeInfo.Model = a.Model
	deviceTypeInfo.Supplier = a.Supplier
	deviceTypeInfo.Pic = a.Pic
	deviceTypeInfo.Description = a.Description
	return nil
}
