package digital

import (
	"context"
	"ifms/internal/config"

	"ifms/internal/mods/digital/api"
	"ifms/internal/mods/digital/schema"

	"github.com/gin-gonic/gin"
	"gorm.io/gorm"
)

type DIGITAL struct {
	DB         *gorm.DB
	DiagramAPI *api.Diagram
}

func (a *DIGITAL) AutoMigrate(ctx context.Context) error {
	return a.DB.AutoMigrate(new(schema.Diagram))
}

func (a *DIGITAL) Init(ctx context.Context) error {
	if config.C.Storage.DB.AutoMigrate {
		if err := a.AutoMigrate(ctx); err != nil {
			return err
		}
	}
	return nil
}

func (a *DIGITAL) RegisterV1Routers(ctx context.Context, v1 *gin.RouterGroup) error {
	v1 = v1.Group("digital")
	diagram := v1.Group("diagrams")
	{
		diagram.GET("", a.DiagramAPI.Query)
		diagram.GET(":id", a.DiagramAPI.Get)
		diagram.POST("/create", a.DiagramAPI.Create)
		diagram.POST("/update/:id", a.DiagramAPI.Update)
		diagram.POST("/delete/:id", a.DiagramAPI.Delete)
	}
	return nil
}

func (a *DIGITAL) Release(ctx context.Context) error {
	return nil
}
