package schema

import (
	"encoding/json"
	"time"

	"ifms/pkg/util"
)

// 图纸管理
type Diagram struct {
	ID          int64           `json:"id" gorm:"primaryKey;comment:Unique ID;"`    // Unique ID
	CompanyCode string          `json:"company_code" gorm:"size:255;comment:公司代码;"` // 公司代码
	DiagramUrl  string          `json:"diagram_url" gorm:"size:500;comment:图纸地址;"`  // 图纸地址
	ThingsInfo  json.RawMessage `json:"things_info" gorm:"type:json;comment:物联信息;"` // 物联信息
	DiagramName string          `json:"diagram_name" gorm:"size:500;comment:图纸名称;"` // 图纸名称
	AnimateMsg  json.RawMessage `json:"animate_msg" gorm:"type:json;comment:动画信息;"` // 动画信息
	DiagramType int             `json:"diagram_type" gorm:"comment:图纸类型;"`          // 图纸类型
	IsShow      bool            `json:"is_show" gorm:"comment:是否显示;"`
	CreatedAt   time.Time       `json:"created_at" gorm:"index;comment:Create time;"` // Create time
	UpdatedAt   time.Time       `json:"updated_at" gorm:"index;comment:Update time;"` // Update time
}

// Defining the query parameters for the `Diagram` struct.
type DiagramQueryParam struct {
	util.PaginationParam
}

// Defining the query options for the `Diagram` struct.
type DiagramQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `Diagram` struct.
type DiagramQueryResult struct {
	Data       Diagrams
	PageResult *util.PaginationResult
}

// Defining the slice of `Diagram` struct.
type Diagrams []*Diagram

// Defining the data structure for creating a `Diagram` struct.
type DiagramForm struct {
	CompanyCode string          `json:"company_code"` // 公司代码
	DiagramUrl  string          `json:"diagram_url"`  // 图纸地址
	ThingsInfo  json.RawMessage `json:"things_info"`  // 物联信息
	DiagramName string          `json:"diagram_name"` // 图纸名称
	AnimateMsg  json.RawMessage `json:"animate_msg"`  // 动画信息
	IsShow      bool            `json:"is_show"`      // 是否显示
	DiagramType int             `json:"diagram_type"` // 图纸类型
}

// A validation function for the `DiagramForm` struct.
func (a *DiagramForm) Validate() error {
	return nil
}

// Convert `DiagramForm` to `Diagram` object.
func (a *DiagramForm) FillTo(diagram *Diagram) error {
	diagram.CompanyCode = a.CompanyCode
	diagram.DiagramUrl = a.DiagramUrl
	diagram.ThingsInfo = a.ThingsInfo
	diagram.DiagramName = a.DiagramName
	diagram.AnimateMsg = a.AnimateMsg
	diagram.DiagramType = a.DiagramType
	diagram.IsShow = a.IsShow
	return nil
}
