package dal

import (
	"context"

	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get env monitor device fish pond storage instance
func GetEnvMonitorDeviceFishPondDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.EnvMonitorDeviceFishPond))
}

// 环境监测设备-鱼池关联表
type EnvMonitorDeviceFishPond struct {
	DB *gorm.DB
}

// Query env monitor device fish ponds from the database based on the provided parameters and options.
func (a *EnvMonitorDeviceFishPond) Query(ctx context.Context, params schema.EnvMonitorDeviceFishPondQueryParam, opts ...schema.EnvMonitorDeviceFishPondQueryOptions) (*schema.EnvMonitorDeviceFishPondQueryResult, error) {
	var opt schema.EnvMonitorDeviceFishPondQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	db := GetEnvMonitorDeviceFishPondDB(ctx, a.DB)

	var list schema.EnvMonitorDeviceFishPonds
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.EnvMonitorDeviceFishPondQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified env monitor device fish pond from the database.
func (a *EnvMonitorDeviceFishPond) Get(ctx context.Context, id string, opts ...schema.EnvMonitorDeviceFishPondQueryOptions) (*schema.EnvMonitorDeviceFishPond, error) {
	var opt schema.EnvMonitorDeviceFishPondQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.EnvMonitorDeviceFishPond)
	ok, err := util.FindOne(ctx, GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified env monitor device fish pond exists in the database.
func (a *EnvMonitorDeviceFishPond) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new env monitor device fish pond.
func (a *EnvMonitorDeviceFishPond) Create(ctx context.Context, item *schema.EnvMonitorDeviceFishPond) error {
	result := GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified env monitor device fish pond in the database.
func (a *EnvMonitorDeviceFishPond) Update(ctx context.Context, item *schema.EnvMonitorDeviceFishPond) error {
	result := GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified env monitor device fish pond from the database.
func (a *EnvMonitorDeviceFishPond) Delete(ctx context.Context, id string) error {
	result := GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.EnvMonitorDeviceFishPond))
	return errors.WithStack(result.Error)
}

func (a *EnvMonitorDeviceFishPond) DeleteByDeviceID(ctx context.Context, deviceID int64) error {
	result := GetEnvMonitorDeviceFishPondDB(ctx, a.DB).Where("env_monitor_device_id=?", deviceID).Delete(new(schema.EnvMonitorDeviceFishPond))
	return errors.WithStack(result.Error)
}
