package dal

import (
	"context"

	"ifms/internal/consts"
	"ifms/internal/mods/envmonitor/schema"
	"ifms/pkg/errors"
	"ifms/pkg/gormx"
	"ifms/pkg/util"

	"gorm.io/gorm"
)

// Get env monitor device type info storage instance
func GetEnvMonitorDeviceTypeInfoDB(ctx context.Context, defDB *gorm.DB) *gorm.DB {
	return util.GetDB(ctx, defDB).Model(new(schema.EnvMonitorDeviceTypeInfo))
}

// 环境监测设备类型信息
type EnvMonitorDeviceTypeInfo struct {
	DB *gorm.DB
}

// Query env monitor device type infos from the database based on the provided parameters and options.
func (a *EnvMonitorDeviceTypeInfo) Query(ctx context.Context, params schema.EnvMonitorDeviceTypeInfoQueryParam, opts ...schema.EnvMonitorDeviceTypeInfoQueryOptions) (*schema.EnvMonitorDeviceTypeInfoQueryResult, error) {
	var opt schema.EnvMonitorDeviceTypeInfoQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}
	ctx = gormx.WithTenantID(ctx, consts.DefaultTenantID)
	db := GetEnvMonitorDeviceTypeInfoDB(ctx, a.DB)

	var list schema.EnvMonitorDeviceTypeInfos
	pageResult, err := util.WrapPageQuery(ctx, db, params.PaginationParam, opt.QueryOptions, &list)
	if err != nil {
		return nil, errors.WithStack(err)
	}

	queryResult := &schema.EnvMonitorDeviceTypeInfoQueryResult{
		PageResult: pageResult,
		Data:       list,
	}
	return queryResult, nil
}

// Get the specified env monitor device type info from the database.
func (a *EnvMonitorDeviceTypeInfo) Get(ctx context.Context, id string, opts ...schema.EnvMonitorDeviceTypeInfoQueryOptions) (*schema.EnvMonitorDeviceTypeInfo, error) {
	var opt schema.EnvMonitorDeviceTypeInfoQueryOptions
	if len(opts) > 0 {
		opt = opts[0]
	}

	item := new(schema.EnvMonitorDeviceTypeInfo)
	ok, err := util.FindOne(ctx, GetEnvMonitorDeviceTypeInfoDB(ctx, a.DB).Where("id=?", id), opt.QueryOptions, item)
	if err != nil {
		return nil, errors.WithStack(err)
	} else if !ok {
		return nil, nil
	}
	return item, nil
}

// Exists checks if the specified env monitor device type info exists in the database.
func (a *EnvMonitorDeviceTypeInfo) Exists(ctx context.Context, id string) (bool, error) {
	ok, err := util.Exists(ctx, GetEnvMonitorDeviceTypeInfoDB(ctx, a.DB).Where("id=?", id))
	return ok, errors.WithStack(err)
}

// Create a new env monitor device type info.
func (a *EnvMonitorDeviceTypeInfo) Create(ctx context.Context, item *schema.EnvMonitorDeviceTypeInfo) error {
	result := GetEnvMonitorDeviceTypeInfoDB(ctx, a.DB).Create(item)
	return errors.WithStack(result.Error)
}

// Update the specified env monitor device type info in the database.
func (a *EnvMonitorDeviceTypeInfo) Update(ctx context.Context, item *schema.EnvMonitorDeviceTypeInfo) error {
	result := GetEnvMonitorDeviceTypeInfoDB(ctx, a.DB).Where("id=?", item.ID).Select("*").Omit("created_at").Updates(item)
	return errors.WithStack(result.Error)
}

// Delete the specified env monitor device type info from the database.
func (a *EnvMonitorDeviceTypeInfo) Delete(ctx context.Context, id string) error {
	result := GetEnvMonitorDeviceTypeInfoDB(ctx, a.DB).Where("id=?", id).Delete(new(schema.EnvMonitorDeviceTypeInfo))
	return errors.WithStack(result.Error)
}
