package envmonitor

import (
	"context"
	"ifms/internal/config"

	"ifms/internal/mods/envmonitor/api"
	"ifms/internal/mods/envmonitor/schema"

	"github.com/gin-gonic/gin"
	"gorm.io/gorm"
)

type EnvMonitor struct {
	DB                               *gorm.DB
	EnvMonitorDeviceTypeInfoAPI      *api.EnvMonitorDeviceTypeInfo
	EnvMonitorDeviceAPI              *api.EnvMonitorDevice
	EnvMonitorDeviceFishPondAPI      *api.EnvMonitorDeviceFishPond
	EnvMonitorDeviceWarningConfigAPI *api.EnvMonitorDeviceWarningConfig
	EnvMonitorDeviceSensorAPI        *api.EnvMonitorDeviceSensor
	EnvMonitorDeviceWarningLogAPI    *api.EnvMonitorDeviceWarningLog
}

func (a *EnvMonitor) AutoMigrate(ctx context.Context) error {
	return a.DB.AutoMigrate(new(schema.EnvMonitorDeviceTypeInfo), new(schema.EnvMonitorDevice), new(schema.EnvMonitorDeviceFishPond), new(schema.EnvMonitorDeviceWarningConfig), new(schema.EnvMonitorDeviceSensor), new(schema.EnvMonitorDeviceWarningLog))
}

func (a *EnvMonitor) Init(ctx context.Context) error {
	if config.C.Storage.DB.AutoMigrate {
		if err := a.AutoMigrate(ctx); err != nil {
			return err
		}
	}
	return nil
}

func (a *EnvMonitor) RegisterV1Routers(ctx context.Context, v1 *gin.RouterGroup) error {
	v1 = v1.Group("envmonitor")
	envMonitorDeviceTypeInfo := v1.Group("env-monitor-device-type-infos")
	{
		envMonitorDeviceTypeInfo.GET("", a.EnvMonitorDeviceTypeInfoAPI.Query)
		envMonitorDeviceTypeInfo.GET(":id", a.EnvMonitorDeviceTypeInfoAPI.Get)
		envMonitorDeviceTypeInfo.POST("", a.EnvMonitorDeviceTypeInfoAPI.Create)
		envMonitorDeviceTypeInfo.PUT(":id", a.EnvMonitorDeviceTypeInfoAPI.Update)
		envMonitorDeviceTypeInfo.DELETE(":id", a.EnvMonitorDeviceTypeInfoAPI.Delete)
	}
	envMonitorDevice := v1.Group("env-monitor-devices")
	{
		// 统计设备数量以及状态
		envMonitorDevice.GET("statistics", a.EnvMonitorDeviceAPI.Statistics)
		envMonitorDevice.GET("", a.EnvMonitorDeviceAPI.Query)
		envMonitorDevice.GET(":id", a.EnvMonitorDeviceAPI.Get)
		envMonitorDevice.POST("", a.EnvMonitorDeviceAPI.Create)
		envMonitorDevice.PUT(":id", a.EnvMonitorDeviceAPI.Update)
		// envMonitorDevice.DELETE(":id", a.EnvMonitorDeviceAPI.Delete)

	}
	envMonitorDeviceWarningConfig :=
		// envMonitorDeviceFishPond := v1.Group("env-monitor-device-fish-ponds")
		// {
		// 	envMonitorDeviceFishPond.GET("", a.EnvMonitorDeviceFishPondAPI.Query)
		// 	envMonitorDeviceFishPond.GET(":id", a.EnvMonitorDeviceFishPondAPI.Get)
		// 	envMonitorDeviceFishPond.POST("", a.EnvMonitorDeviceFishPondAPI.Create)
		// 	envMonitorDeviceFishPond.PUT(":id", a.EnvMonitorDeviceFishPondAPI.Update)
		// 	envMonitorDeviceFishPond.DELETE(":id", a.EnvMonitorDeviceFishPondAPI.Delete)
		// }
		v1.Group("env-monitor-device-warning-configs")
	{
		envMonitorDeviceWarningConfig.GET("", a.EnvMonitorDeviceWarningConfigAPI.Query)
		envMonitorDeviceWarningConfig.GET(":id", a.EnvMonitorDeviceWarningConfigAPI.Get)
		envMonitorDeviceWarningConfig.POST("", a.EnvMonitorDeviceWarningConfigAPI.Create)
		envMonitorDeviceWarningConfig.PUT(":id", a.EnvMonitorDeviceWarningConfigAPI.Update)
		envMonitorDeviceWarningConfig.DELETE(":id", a.EnvMonitorDeviceWarningConfigAPI.Delete)
	}
	envMonitorDeviceWarningLog :=
		// envMonitorDeviceSensor := v1.Group("env-monitor-device-sensors")
		// {
		// 	envMonitorDeviceSensor.GET("", a.EnvMonitorDeviceSensorAPI.Query)
		// 	envMonitorDeviceSensor.GET(":id", a.EnvMonitorDeviceSensorAPI.Get)
		// 	envMonitorDeviceSensor.POST("", a.EnvMonitorDeviceSensorAPI.Create)
		// 	envMonitorDeviceSensor.PUT(":id", a.EnvMonitorDeviceSensorAPI.Update)
		// 	envMonitorDeviceSensor.DELETE(":id", a.EnvMonitorDeviceSensorAPI.Delete)
		// }
		v1.Group("env-monitor-device-warning-logs")
	{
		envMonitorDeviceWarningLog.GET("", a.EnvMonitorDeviceWarningLogAPI.Query)
		// envMonitorDeviceWarningLog.GET(":id", a.EnvMonitorDeviceWarningLogAPI.Get)
		envMonitorDeviceWarningLog.POST("", a.EnvMonitorDeviceWarningLogAPI.Create)
		// envMonitorDeviceWarningLog.PUT(":id", a.EnvMonitorDeviceWarningLogAPI.Update)
		// envMonitorDeviceWarningLog.DELETE(":id", a.EnvMonitorDeviceWarningLogAPI.Delete)
		envMonitorDeviceWarningLog.GET("export", a.EnvMonitorDeviceWarningLogAPI.Export)
	}

	return nil
}

func (a *EnvMonitor) Release(ctx context.Context) error {
	return nil
}
