package schema

import (
	"time"

	"ifms/pkg/util"
)

// 环境监测设备-鱼池关联表
type EnvMonitorDeviceFishPond struct {
	ID                 int64     `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`                            // Unique ID
	TenantID           string    `json:"tenant_id" gorm:"index;comment:租户ID;"`                                       // 租户ID
	EnvMonitorDeviceID int64     `json:"device_id" gorm:"index;foreignKey:ID;references:device;comment:环境监测设备ID;"`   // 环境监测设备ID
	FishPondID         int64     `json:"fish_pond_id" gorm:"index;foreignKey:ID;references:fish_pond;comment:鱼池ID;"` // 鱼池ID
	FishPondName       string    `json:"fish_pond_name" gorm:"comment:鱼池名称;"`                                        // 鱼池名称
	CreatedAt          time.Time `json:"created_at" gorm:"index;comment:Create time;"`                               // Create time
	UpdatedAt          time.Time `json:"updated_at" gorm:"index;comment:Update time;"`                               // Update time
}

// Defining the query parameters for the `EnvMonitorDeviceFishPond` struct.
type EnvMonitorDeviceFishPondQueryParam struct {
	util.PaginationParam
}

// Defining the query options for the `EnvMonitorDeviceFishPond` struct.
type EnvMonitorDeviceFishPondQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `EnvMonitorDeviceFishPond` struct.
type EnvMonitorDeviceFishPondQueryResult struct {
	Data       EnvMonitorDeviceFishPonds
	PageResult *util.PaginationResult
}

// Defining the slice of `EnvMonitorDeviceFishPond` struct.
type EnvMonitorDeviceFishPonds []*EnvMonitorDeviceFishPond

// Defining the data structure for creating a `EnvMonitorDeviceFishPond` struct.
type EnvMonitorDeviceFishPondForm struct {
	TenantID           string `json:"tenant_id" binding:"required"`    // 租户ID
	EnvMonitorDeviceID int64  `json:"device_id" binding:"required"`    // 环境监测设备ID
	FishPondID         int64  `json:"fish_pond_id" binding:"required"` // 鱼池ID
}

// A validation function for the `EnvMonitorDeviceFishPondForm` struct.
func (a *EnvMonitorDeviceFishPondForm) Validate() error {
	return nil
}

// Convert `EnvMonitorDeviceFishPondForm` to `EnvMonitorDeviceFishPond` object.
func (a *EnvMonitorDeviceFishPondForm) FillTo(envMonitorDeviceFishPond *EnvMonitorDeviceFishPond) error {
	envMonitorDeviceFishPond.EnvMonitorDeviceID = a.EnvMonitorDeviceID
	envMonitorDeviceFishPond.FishPondID = a.FishPondID
	envMonitorDeviceFishPond.TenantID = a.TenantID
	return nil
}
