package schema

import (
	"time"

	"ifms/pkg/util"
)

// 设备类型（传感器类型 1水温传感器、2溶解氧传感器、3氨氮传感器、4PH传感器、5电导率传感器、6视频监控）
const (
	EnvMonitorDeviceSensorTypeWaterTemperature = 1 // 水温传感器
	EnvMonitorDeviceSensorTypeDissolvedOxygen  = 2 // 溶解氧传感器
	EnvMonitorDeviceSensorTypeAmmonia          = 3 // 氨氮传感器
	EnvMonitorDeviceSensorTypePH               = 4 // PH传感器
	EnvMonitorDeviceSensorTypeConductivity     = 5 // 电导率传感器
	EnvMonitorDeviceSensorTypeVideo            = 6 // 视频监控
)

// 设备传感器表
type EnvMonitorDeviceSensor struct {
	ID                 int64     `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`                               // Unique ID
	EnvMonitorDeviceID int64     `json:"env_monitor_device_id" gorm:"index;comment:环境监测设备ID;"`                          // 环境监测设备ID
	TenantID           string    `json:"tenant_id" gorm:"size:36;comment:租户ID;"`                                        // 租户ID
	Name               string    `json:"name" gorm:"size:64;comment:传感器名称;"`                                            // 传感器名称
	Type               int64     `json:"type" gorm:"size:64;comment:传感器类型 1水温传感器、2溶解氧传感器、3氨氮传感器、4PH传感器、5电导率传感器 6视频监控;"` // 传感器类型
	Status             bool      `json:"status" gorm:"comment:传感器状态（true=正常，false=异常）;"`                                // 传感器状态（true=正常，false=异常）
	CurrentValue       float64   `json:"current_value" gorm:"comment:当前值;"`                                             // 当前值
	CreatedAt          time.Time `json:"created_at" gorm:"index;comment:Create time;"`                                  // Create time
	UpdatedAt          time.Time `json:"updated_at" gorm:"index;comment:Update time;"`                                  // Update time
}

// Defining the query parameters for the `EnvMonitorDeviceSensor` struct.
type EnvMonitorDeviceSensorQueryParam struct {
	util.PaginationParam
}

// Defining the query options for the `EnvMonitorDeviceSensor` struct.
type EnvMonitorDeviceSensorQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `EnvMonitorDeviceSensor` struct.
type EnvMonitorDeviceSensorQueryResult struct {
	Data       EnvMonitorDeviceSensors
	PageResult *util.PaginationResult
}

// Defining the slice of `EnvMonitorDeviceSensor` struct.
type EnvMonitorDeviceSensors []*EnvMonitorDeviceSensor

// Defining the data structure for creating a `EnvMonitorDeviceSensor` struct.
type EnvMonitorDeviceSensorForm struct {
	EnvMonitorDeviceID int64   `json:"env_monitor_device_id" binding:"required"` // 环境监测设备ID
	TenantID           string  `json:"tenant_id" binding:"required,max=36"`      // 租户ID
	Status             bool    `json:"status" binding:"required"`                // 传感器状态（true=正常，false=异常）
	CurrentValue       float64 `json:"current_value" binding:"required"`         // 当前值
	Type               int64   `json:"type" binding:"required,max=64"`           // 传感器类型
	Name               string  `json:"name" binding:"required,max=64"`           // 传感器名称
}

// A validation function for the `EnvMonitorDeviceSensorForm` struct.
func (a *EnvMonitorDeviceSensorForm) Validate() error {
	return nil
}

// Convert `EnvMonitorDeviceSensorForm` to `EnvMonitorDeviceSensor` object.
func (a *EnvMonitorDeviceSensorForm) FillTo(envMonitorDeviceSensor *EnvMonitorDeviceSensor) error {
	envMonitorDeviceSensor.EnvMonitorDeviceID = a.EnvMonitorDeviceID
	envMonitorDeviceSensor.TenantID = a.TenantID
	envMonitorDeviceSensor.Status = a.Status
	envMonitorDeviceSensor.CurrentValue = a.CurrentValue
	envMonitorDeviceSensor.Type = a.Type
	envMonitorDeviceSensor.Name = a.Name
	return nil
}
