package schema

import (
	"time"

	"ifms/pkg/util"
)

// 环境监测设备类型信息
type EnvMonitorDeviceTypeInfo struct {
	ID          int64     `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`       // Unique ID
	TenantID    string    `json:"tenant_id" gorm:"index;comment:租户ID;"`                  // 租户ID
	Type        int       `json:"type" gorm:"comment:环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）;"` // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
	Model       string    `json:"model" gorm:"size:255;comment:设备型号;"`                   // 设备型号
	Supplier    string    `json:"supplier" gorm:"size:255;comment:供应商;"`                 // 供应商
	Pic         string    `json:"pic" gorm:"type:text;comment:图片地址;"`                    // 图片地址
	Description string    `json:"description" gorm:"type:text;comment:简介;"`              // 简介
	CreatedAt   time.Time `json:"created_at" gorm:"index;comment:Create time;"`          // Create time
	UpdatedAt   time.Time `json:"updated_at" gorm:"index;comment:Update time;"`          // Update time
}

// Defining the query parameters for the `EnvMonitorDeviceTypeInfo` struct.
type EnvMonitorDeviceTypeInfoQueryParam struct {
	util.PaginationParam
}

// Defining the query options for the `EnvMonitorDeviceTypeInfo` struct.
type EnvMonitorDeviceTypeInfoQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `EnvMonitorDeviceTypeInfo` struct.
type EnvMonitorDeviceTypeInfoQueryResult struct {
	Data       EnvMonitorDeviceTypeInfos
	PageResult *util.PaginationResult
}

// Defining the slice of `EnvMonitorDeviceTypeInfo` struct.
type EnvMonitorDeviceTypeInfos []*EnvMonitorDeviceTypeInfo

// Defining the data structure for creating a `EnvMonitorDeviceTypeInfo` struct.
type EnvMonitorDeviceTypeInfoForm struct {
	TenantID    string `json:"tenant_id"`                           // 租户ID
	Type        int    `json:"type" binding:"required"`             // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
	Model       string `json:"model" binding:"required,max=255"`    // 设备型号
	Supplier    string `json:"supplier" binding:"required,max=255"` // 供应商
	Pic         string `json:"pic" binding:"required"`              // 图片地址
	Description string `json:"description" binding:"required"`      // 简介
}

// A validation function for the `EnvMonitorDeviceTypeInfoForm` struct.
func (a *EnvMonitorDeviceTypeInfoForm) Validate() error {
	return nil
}

// Convert `EnvMonitorDeviceTypeInfoForm` to `EnvMonitorDeviceTypeInfo` object.
func (a *EnvMonitorDeviceTypeInfoForm) FillTo(envMonitorDeviceTypeInfo *EnvMonitorDeviceTypeInfo) error {
	envMonitorDeviceTypeInfo.Type = a.Type
	envMonitorDeviceTypeInfo.Model = a.Model
	envMonitorDeviceTypeInfo.Supplier = a.Supplier
	envMonitorDeviceTypeInfo.Pic = a.Pic
	envMonitorDeviceTypeInfo.Description = a.Description
	envMonitorDeviceTypeInfo.TenantID = a.TenantID
	return nil
}
