package schema

import (
	"time"

	"ifms/pkg/util"
)

// 环境监测设备报警范围配置表
type EnvMonitorDeviceWarningConfig struct {
	ID                 int64     `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`       // Unique ID
	TenantID           string    `json:"tenant_id" gorm:"index;comment:租户ID;"`                  // 租户ID
	Type               int       `json:"type" gorm:"comment:环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）;"` // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
	EnvMonitorDeviceID int64     `json:"env_monitor_device_id" gorm:"index;comment:环境监测设备ID;"`  // 环境监测设备ID
	WaterTempMin       float64   `json:"water_temp_min" gorm:"comment:水温正常最小值(℃);"`             // 水温正常最小值(℃)
	WaterTempMax       float64   `json:"water_temp_max" gorm:"comment:水温正常最大值(℃);"`             // 水温正常最大值(℃)
	DissolvedOxygenMin float64   `json:"dissolved_oxygen_min" gorm:"comment:溶解氧正常最小值(mg/l);"`   // 溶解氧正常最小值(mg/l)
	DissolvedOxygenMax float64   `json:"dissolved_oxygen_max" gorm:"comment:溶解氧正常最大值(mg/l);"`   // 溶解氧正常最大值(mg/l)
	AmmoniaNitrogenMin float64   `json:"ammonia_nitrogen_min" gorm:"comment:氨氮正常最小值(mg/l);"`    // 氨氮正常最小值(mg/l)
	AmmoniaNitrogenMax float64   `json:"ammonia_nitrogen_max" gorm:"comment:氨氮正常最大值(mg/l);"`    // 氨氮正常最大值(mg/l)
	PHMin              float64   `json:"ph_min" gorm:"comment:PH值正常最小值;"`                       // PH值正常最小值
	PHMax              float64   `json:"ph_max" gorm:"comment:PH值正常最大值;"`                       // PH值正常最大值
	ConductivityMin    float64   `json:"conductivity_min" gorm:"comment:电导率正常最小值(μS);"`         // 电导率正常最小值(μS)
	ConductivityMax    float64   `json:"conductivity_max" gorm:"comment:电导率正常最大值(μS);"`         // 电导率正常最大值(μS)
	CreatedAt          time.Time `json:"created_at" gorm:"index;comment:Create time;"`          // Create time
	UpdatedAt          time.Time `json:"updated_at" gorm:"index;comment:Update time;"`          // Update time
}

// Defining the query parameters for the `EnvMonitorDeviceWarningConfig` struct.
type EnvMonitorDeviceWarningConfigQueryParam struct {
	util.PaginationParam
	Type int `form:"type"` // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
}

// Defining the query options for the `EnvMonitorDeviceWarningConfig` struct.
type EnvMonitorDeviceWarningConfigQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `EnvMonitorDeviceWarningConfig` struct.
type EnvMonitorDeviceWarningConfigQueryResult struct {
	Data       EnvMonitorDeviceWarningConfigs
	PageResult *util.PaginationResult
}

// Defining the slice of `EnvMonitorDeviceWarningConfig` struct.
type EnvMonitorDeviceWarningConfigs []*EnvMonitorDeviceWarningConfig

// Defining the data structure for creating a `EnvMonitorDeviceWarningConfig` struct.
type EnvMonitorDeviceWarningConfigForm struct {
	TenantID           string  `json:"tenant_id" gorm:"index;comment:租户ID;"`                  // 租户ID
	Type               int     `json:"type" gorm:"comment:环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）;"` // 环境监测设备类型（1水质监测仪、2水下视频监控、3车间视频监控）
	EnvMonitorDeviceID int64   `json:"env_monitor_device_id"`                                 // 环境监测设备ID
	WaterTempMin       float64 `json:"water_temp_min"`                                        // 水温正常最小值(℃)
	WaterTempMax       float64 `json:"water_temp_max"`                                        // 水温正常最大值(℃)
	DissolvedOxygenMin float64 `json:"dissolved_oxygen_min"`                                  // 溶解氧正常最小值(mg/l)
	DissolvedOxygenMax float64 `json:"dissolved_oxygen_max"`                                  // 溶解氧正常最大值(mg/l)
	AmmoniaNitrogenMin float64 `json:"ammonia_nitrogen_min"`                                  // 氨氮正常最小值(mg/l)
	AmmoniaNitrogenMax float64 `json:"ammonia_nitrogen_max"`                                  // 氨氮正常最大值(mg/l)
	PHMin              float64 `json:"ph_min"`                                                // PH值正常最小值
	PHMax              float64 `json:"ph_max"`                                                // PH值正常最大值
	ConductivityMin    float64 `json:"conductivity_min"`                                      // 电导率正常最小值(μS)
	ConductivityMax    float64 `json:"conductivity_max"`                                      // 电导率正常最大值(μS)
}

// A validation function for the `EnvMonitorDeviceWarningConfigForm` struct.
func (a *EnvMonitorDeviceWarningConfigForm) Validate() error {
	return nil
}

// Convert `EnvMonitorDeviceWarningConfigForm` to `EnvMonitorDeviceWarningConfig` object.
func (a *EnvMonitorDeviceWarningConfigForm) FillTo(envMonitorDeviceWarningConfig *EnvMonitorDeviceWarningConfig) error {
	envMonitorDeviceWarningConfig.EnvMonitorDeviceID = a.EnvMonitorDeviceID
	envMonitorDeviceWarningConfig.WaterTempMin = a.WaterTempMin
	envMonitorDeviceWarningConfig.WaterTempMax = a.WaterTempMax
	envMonitorDeviceWarningConfig.DissolvedOxygenMin = a.DissolvedOxygenMin
	envMonitorDeviceWarningConfig.DissolvedOxygenMax = a.DissolvedOxygenMax
	envMonitorDeviceWarningConfig.AmmoniaNitrogenMin = a.AmmoniaNitrogenMin
	envMonitorDeviceWarningConfig.AmmoniaNitrogenMax = a.AmmoniaNitrogenMax
	envMonitorDeviceWarningConfig.PHMin = a.PHMin
	envMonitorDeviceWarningConfig.PHMax = a.PHMax
	envMonitorDeviceWarningConfig.ConductivityMin = a.ConductivityMin
	envMonitorDeviceWarningConfig.ConductivityMax = a.ConductivityMax
	envMonitorDeviceWarningConfig.TenantID = a.TenantID
	envMonitorDeviceWarningConfig.Type = a.Type
	return nil
}
