package schema

import (
	"time"

	"ifms/pkg/util"
)

// 环境监测设备报警日志
type EnvMonitorDeviceWarningLog struct {
	ID                       int64     `json:"id" gorm:"size:20;primaryKey;comment:Unique ID;"`              // Unique ID
	TenantID                 string    `json:"tenant_id" gorm:"size:32;comment:租户ID;"`                       // 租户ID
	EnvMonitorDeviceID       int64     `json:"env_monitor_device_id" gorm:"size:32;comment:设备ID;"`           // 设备ID
	EnvMonitorDeviceSensorID int64     `json:"env_monitor_device_sensor_id" gorm:"size:32;comment:设备传感器ID;"` // 设备传感器ID
	DeviceModel              string    `json:"device_model" gorm:"size:64;comment:设备型号;"`                    // 设备型号
	DeviceName               string    `json:"device_name" gorm:"size:64;comment:设备名称;"`                     // 设备名称
	RelatedPondIds           string    `json:"related_pond_ids" gorm:"type:text;comment:关联鱼池/车间ID;"`         // 关联鱼池/车间ID
	RelatedPond              string    `json:"related_pond" gorm:"type:text;comment:关联鱼池/车间;"`               // 关联鱼池/车间
	Type                     string    `json:"type" gorm:"size:255;comment:报警指标;"`                           // 报警指标
	StandardRange            string    `json:"standard_range" gorm:"size:255;comment:参考数值范围;"`               // 参考数值范围
	CurrentValue             float64   `json:"current_value" gorm:"type:decimal(5,2);comment:当前数值;"`         // 当前数值
	WarningType              int       `json:"warning_type" gorm:"comment:1偏高 2偏低;"`                         // 1偏高 2偏低
	IsOk                     bool      `json:"is_ok" gorm:"comment:是否正常;"`                                   // 是否正常
	CreatedAt                time.Time `json:"created_at" gorm:"index;comment:Create time;"`                 // Create time
	UpdatedAt                time.Time `json:"updated_at" gorm:"index;comment:Update time;"`                 // Update time
}

// Defining the query parameters for the `EnvMonitorDeviceWarningLog` struct.
type EnvMonitorDeviceWarningLogQueryParam struct {
	util.PaginationParam
	DeviceName string `form:"device_name"` // 设备名称
	StartTime  string `form:"start_time"`  // 开始时间
	EndTime    string `form:"end_time"`    // 结束时间
	IsOK       *bool  `form:"is_ok"`       // 是否正常
	Ids        string `form:"ids"`         // 设备ID
	Type       string `form:"type"`        // 报警指标
}

// Defining the query options for the `EnvMonitorDeviceWarningLog` struct.
type EnvMonitorDeviceWarningLogQueryOptions struct {
	util.QueryOptions
}

// Defining the query result for the `EnvMonitorDeviceWarningLog` struct.
type EnvMonitorDeviceWarningLogQueryResult struct {
	Data       EnvMonitorDeviceWarningLogs
	PageResult *util.PaginationResult
}

// Defining the slice of `EnvMonitorDeviceWarningLog` struct.
type EnvMonitorDeviceWarningLogs []*EnvMonitorDeviceWarningLog

// Defining the data structure for creating a `EnvMonitorDeviceWarningLog` struct.
type EnvMonitorDeviceWarningLogForm struct {
	TenantID                 string  `json:"tenant_id"`                                       // 租户ID
	EnvMonitorDeviceID       int64   `json:"env_monitor_device_id" binding:"required"`        // 设备ID
	EnvMonitorDeviceSensorID int64   `json:"env_monitor_device_sensor_id" binding:"required"` // 设备传感器ID
	DeviceModel              string  `json:"device_model" binding:"required,max=64"`          // 设备型号
	DeviceName               string  `json:"device_name" binding:"required,max=64"`           // 设备名称
	RelatedPondIds           string  `json:"related_pond_ids" binding:"required"`             // 关联鱼池/车间ID
	RelatedPond              string  `json:"related_pond" binding:"required"`                 // 关联鱼池/车间
	Type                     string  `json:"type"`                                            // 报警指标
	StandardRange            string  `json:"standard_range"`                                  // 参考数值范围
	CurrentValue             float64 `json:"current_value"`                                   // 当前数值
}

// A validation function for the `EnvMonitorDeviceWarningLogForm` struct.
func (a *EnvMonitorDeviceWarningLogForm) Validate() error {
	return nil
}

// Convert `EnvMonitorDeviceWarningLogForm` to `EnvMonitorDeviceWarningLog` object.
func (a *EnvMonitorDeviceWarningLogForm) FillTo(envMonitorDeviceWarningLog *EnvMonitorDeviceWarningLog) error {
	envMonitorDeviceWarningLog.TenantID = a.TenantID
	envMonitorDeviceWarningLog.EnvMonitorDeviceID = a.EnvMonitorDeviceID
	envMonitorDeviceWarningLog.EnvMonitorDeviceSensorID = a.EnvMonitorDeviceSensorID
	envMonitorDeviceWarningLog.DeviceModel = a.DeviceModel
	envMonitorDeviceWarningLog.DeviceName = a.DeviceName
	envMonitorDeviceWarningLog.RelatedPondIds = a.RelatedPondIds
	envMonitorDeviceWarningLog.RelatedPond = a.RelatedPond
	envMonitorDeviceWarningLog.Type = a.Type
	envMonitorDeviceWarningLog.StandardRange = a.StandardRange
	envMonitorDeviceWarningLog.CurrentValue = a.CurrentValue
	return nil
}

type EnvMonitorDeviceWarningLogExport struct {
	Url string `json:"url"`
}
