package mods

import (
	"context"

	"ifms/internal/mods/digital"
	"ifms/internal/mods/rbac"

	"ifms/internal/mods/autocontrol"

	"ifms/internal/mods/envmonitor"

	"github.com/gin-gonic/gin"
	"github.com/google/wire"
)

const (
	apiPrefix = "/api/"
)

// Collection of wire providers
var Set = wire.NewSet(
	wire.Struct(new(Mods), "*"),
	rbac.Set,
	digital.Set,
	autocontrol.Set,
	envmonitor.Set,
)

type Mods struct {
	RBAC        *rbac.RBAC
	DIGITAL     *digital.DIGITAL
	AutoControl *autocontrol.AutoControl
	EnvMonitor  *envmonitor.EnvMonitor
}

func (a *Mods) Init(ctx context.Context) error {
	if err := a.RBAC.Init(ctx); err != nil {
		return err
	}
	if err := a.DIGITAL.Init(ctx); err != nil {
		return err
	}
	if err := a.AutoControl.Init(ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.Init(ctx); err != nil {
		return err
	}

	return nil
}

func (a *Mods) RouterPrefixes() []string {
	return []string{
		apiPrefix,
	}
}

func (a *Mods) RegisterRouters(ctx context.Context, e *gin.Engine) error {
	gAPI := e.Group(apiPrefix)
	v1 := gAPI.Group("v1")

	if err := a.RBAC.RegisterV1Routers(ctx, v1); err != nil {
		return err
	}
	if err := a.DIGITAL.RegisterV1Routers(ctx, v1); err != nil {
		return err
	}
	if err := a.AutoControl.RegisterV1Routers(ctx, v1); err != nil {
		return err
	}
	if err := a.EnvMonitor.RegisterV1Routers(ctx, v1); err != nil {
		return err
	}

	return nil
}

func (a *Mods) Release(ctx context.Context) error {
	if err := a.RBAC.Release(ctx); err != nil {
		return err
	}
	if err := a.DIGITAL.Release(ctx); err != nil {
		return err
	}
	if err := a.AutoControl.Release(
		ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.
		Release(ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.
		Release(ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.
		Release(ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.
		Release(ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.
		Release(ctx); err != nil {
		return err
	}
	if err := a.EnvMonitor.
		Release(ctx); err != nil {
		return err
	}

	return nil
}
